@@ -142,7 +142,7 @@ class ApiInvocationContext:
         self.headers = headers
         self.context = {} if context is None else context
         self.auth_info = {} if auth_info is None else auth_info
-        self.apigw_version = ApiGatewayVersion.V1
+        self.apigw_version = None
         self.api_id = api_id
         self.stage = stage
         self.region_name = None
@@ -168,10 +168,9 @@ class ApiInvocationContext:
         return self._path_with_query_string or self.path
 
     @path_with_query_string.setter
-    def path_with_query_string(self, new_path) -> str:
+    def path_with_query_string(self, new_path: str):
         """Set a custom invocation path with query string (used to handle "../_user_request_/.." paths)."""
         self._path_with_query_string = new_path
-        return new_path
 
     @property
     def integration_uri(self) -> Optional[str]:
@@ -181,12 +180,27 @@ class ApiInvocationContext:
     @property
     def auth_context(self) -> Optional[Dict]:
         if isinstance(self.auth_info, dict):
-            context = self.auth_info.get("context") or {}
+            context = self.auth_info.setdefault("context", {})
             principal = self.auth_info.get("principalId")
             if principal:
                 context["principalId"] = principal
             return context
 
+    @property
+    def auth_identity(self) -> Optional[Dict]:
+        if isinstance(self.auth_info, dict):
+            if self.auth_info.get("identity") is None:
+                self.auth_info["identity"] = {}
+            return self.auth_info["identity"]
+
+    def is_websocket_request(self):
+        upgrade_header = str(self.headers.get("upgrade") or "")
+        return upgrade_header.lower() == "websocket"
+
+    def is_v1(self):
+        """Whether this is an API Gateway v1 request"""
+        return self.apigw_version == ApiGatewayVersion.V1
+
 
 class ProxyListenerApiGateway(ProxyListener):
     def forward_request(self, method, path, data, headers):
@@ -212,6 +226,9 @@ class ProxyListenerApiGateway(ProxyListener):
         if re.match(PATH_REGEX_RESPONSES, path):
             return handle_gateway_responses(method, path, data, headers)
 
+        if re.match(PATH_REGEX_PATH_MAPPINGS, path):
+            return handle_base_path_mappings(method, path, data, headers)
+
         if is_test_invoke_method(method, path):
             # if call is from test_invoke_api then use http_method to find the integration,
             #   as test_invoke_api makes a POST call to request the test invocation
@@ -247,8 +264,6 @@ class ProxyListenerApiGateway(ProxyListener):
                 result = handle_accounts(method, path, data, headers)
             elif path.startswith("/vpclinks"):
                 result = handle_vpc_links(method, path, data, headers)
-            elif re.match(PATH_REGEX_PATH_MAPPINGS, path):
-                result = handle_base_path_mappings(method, path, data, headers)
             elif re.match(PATH_REGEX_CLIENT_CERTS, path):
                 result = handle_client_certificates(method, path, data, headers)
 
@@ -404,9 +419,22 @@ def apply_response_parameters(invocation_context: ApiInvocationContext):
     return response
 
 
-def get_api_id_stage_invocation_path(
+def set_api_id_stage_invocation_path(
     invocation_context: ApiInvocationContext,
-) -> Tuple[str, str, str]:
+) -> ApiInvocationContext:
+    # skip if all details are already available
+    values = (
+        invocation_context.api_id,
+        invocation_context.stage,
+        invocation_context.path_with_query_string,
+    )
+    if all(values):
+        return invocation_context
+
+    # skip if this is a websocket request
+    if invocation_context.is_websocket_request():
+        return invocation_context
+
     path = invocation_context.path
     headers = invocation_context.headers
 
@@ -436,14 +464,21 @@ def get_api_id_stage_invocation_path(
         resource_path = resource.get("path")
         relative_path_w_query_params = f"{resource_path}{query_string}"
     else:
-        raise Exception(f"Unable to extract API Gateway details from request: {path} {headers}")
+        raise Exception(
+            f"Unable to extract API Gateway details from request: {path} {dict(headers)}"
+        )
     if api_id:
         # set current region in request thread local, to ensure aws_stack.get_region() works properly
         if getattr(THREAD_LOCAL, "request_context", None) is not None:
             THREAD_LOCAL.request_context.headers[MARKER_APIGW_REQUEST_REGION] = API_REGIONS.get(
                 api_id, ""
             )
-    return api_id, stage, relative_path_w_query_params
+
+    # set details in invocation context
+    invocation_context.api_id = api_id
+    invocation_context.stage = stage
+    invocation_context.path_with_query_string = relative_path_w_query_params
+    return invocation_context
 
 
 def extract_api_id_from_hostname_in_url(hostname: str) -> str:
@@ -454,15 +489,11 @@ def extract_api_id_from_hostname_in_url(hostname: str) -> str:
 
 
 def invoke_rest_api_from_request(invocation_context: ApiInvocationContext):
-    api_id, stage, relative_path_w_query_params = get_api_id_stage_invocation_path(
-        invocation_context
-    )
-    invocation_context.api_id = api_id
-    invocation_context.stage = stage
-    invocation_context.path_with_query_string = relative_path_w_query_params
+    set_api_id_stage_invocation_path(invocation_context)
     try:
         return invoke_rest_api(invocation_context)
     except AuthorizationError as e:
+        api_id = invocation_context.api_id
         return make_error_response("Not authorized to invoke REST API %s: %s" % (api_id, e), 403)
 
 
@@ -471,15 +502,12 @@ def invoke_rest_api(invocation_context: ApiInvocationContext):
     raw_path = invocation_context.path or invocation_path
     method = invocation_context.method
     headers = invocation_context.headers
-    relative_path, query_string_params = extract_query_string_params(path=invocation_path)
 
     # run gateway authorizers for this request
     authorize_invocation(invocation_context)
-    path_map = helpers.get_rest_api_paths(rest_api_id=invocation_context.api_id)
-    try:
-        extracted_path, resource = get_resource_for_path(path=relative_path, path_map=path_map)
-    except Exception:
-        return make_error_response("Unable to find path %s" % raw_path, 404)
+    extracted_path, resource = get_target_resource_details(invocation_context)
+    if not resource:
+        return make_error_response("Unable to find path %s" % invocation_context.path, 404)
 
     api_key_required = resource.get("resourceMethods", {}).get(method, {}).get("apiKeyRequired")
     if not is_api_key_valid(api_key_required, headers, invocation_context.stage):
@@ -494,9 +522,12 @@ def invoke_rest_api(invocation_context: ApiInvocationContext):
         if method == "OPTIONS" and "Origin" in headers:
             # default to returning CORS headers if this is an OPTIONS request
             return get_cors_response(headers)
-        return make_error_response("Unable to find integration for path %s" % raw_path, 404)
+        return make_error_response(
+            "Unable to find integration for: %s %s (%s)" % (method, invocation_path, raw_path),
+            404,
+        )
 
-    res_methods = path_map.get(relative_path, {}).get("resourceMethods", {})
+    res_methods = resource.get("resourceMethods", {})
     meth_integration = res_methods.get(method, {}).get("methodIntegration", {})
     int_responses = meth_integration.get("integrationResponses", {})
     response_templates = int_responses.get("200", {}).get("responseTemplates", {})
@@ -581,7 +612,7 @@ def invoke_rest_api_integration_backend(
 
             # Sample request context:
             # https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-create-api-as-simple-proxy-for-lambda.html#api-gateway-create-api-as-simple-proxy-for-lambda-test
-            request_context = get_lambda_event_request_context(invocation_context)
+            request_context = get_event_request_context(invocation_context)
             stage_variables = (
                 get_stage_variables(api_id, stage)
                 if not is_test_invoke_method(method, path)
@@ -680,7 +711,6 @@ def invoke_rest_api_integration_backend(
 
         elif "states:action/" in uri:
             action = uri.split("/")[-1]
-            payload = {}
 
             if APPLICATION_JSON in integration.get("requestTemplates", {}):
                 payload = apply_request_response_templates(
@@ -693,6 +723,9 @@ def invoke_rest_api_integration_backend(
                 payload = json.loads(data.decode("utf-8"))
             client = aws_stack.connect_to_service("stepfunctions")
 
+            if isinstance(payload.get("input"), dict):
+                payload["input"] = json.dumps(payload["input"])
+
             # Hot fix since step functions local package responses: Unsupported Operation: 'StartSyncExecution'
             method_name = (
                 camel_to_snake_case(action) if action != "StartSyncExecution" else "start_execution"
@@ -705,9 +738,7 @@ def invoke_rest_api_integration_backend(
                 LOG.error(msg)
                 return make_error_response(msg, 400)
 
-            result = method(
-                **payload,
-            )
+            result = method(**payload)
             result = json_safe({k: result[k] for k in result if k not in "ResponseMetadata"})
             response = requests_response(
                 content=result,
@@ -853,30 +884,52 @@ def invoke_rest_api_integration_backend(
     )
 
 
+def get_target_resource_details(invocation_context: ApiInvocationContext) -> Tuple[str, Dict]:
+    """Look up and return the API GW resource (path pattern + resource dict) for the given invocation context."""
+    path_map = helpers.get_rest_api_paths(rest_api_id=invocation_context.api_id)
+    relative_path = invocation_context.invocation_path
+    try:
+        extracted_path, resource = get_resource_for_path(path=relative_path, path_map=path_map)
+        return extracted_path, resource
+    except Exception:
+        return None, None
+
+
+def get_target_resource_method(invocation_context: ApiInvocationContext) -> Optional[Dict]:
+    """Look up and return the API GW resource method for the given invocation context."""
+    _, resource = get_target_resource_details(invocation_context)
+    if not resource:
+        return None
+    methods = resource.get("resourceMethods") or {}
+    method_name = invocation_context.method.upper()
+    method_details = methods.get(method_name) or methods.get("ANY")
+    return method_details
+
+
 def get_stage_variables(api_id: str, stage: str) -> Dict[str, str]:
     if not stage:
-        return
+        return {}
     region_name = [name for name, region in apigateway_backends.items() if api_id in region.apis][0]
     api_gateway_client = aws_stack.connect_to_service("apigateway", region_name=region_name)
     response = api_gateway_client.get_stage(restApiId=api_id, stageName=stage)
     return response.get("variables")
 
 
-def get_lambda_event_request_context(invocation_context: ApiInvocationContext):
+def get_event_request_context(invocation_context: ApiInvocationContext):
     method = invocation_context.method
     path = invocation_context.path
     headers = invocation_context.headers
     integration_uri = invocation_context.integration_uri
     resource_path = invocation_context.resource_path
     resource_id = invocation_context.resource_id
-    auth_context = invocation_context.auth_context
 
-    api_id, stage, relative_path_w_query_params = get_api_id_stage_invocation_path(
-        invocation_context
-    )
+    set_api_id_stage_invocation_path(invocation_context)
     relative_path, query_string_params = extract_query_string_params(
-        path=relative_path_w_query_params
+        path=invocation_context.path_with_query_string
     )
+    api_id = invocation_context.api_id
+    stage = invocation_context.stage
+
     source_ip = headers.get("X-Forwarded-For", ",").split(",")[-2].strip()
     integration_uri = integration_uri or ""
     account_id = integration_uri.split(":lambda:path")[-1].split(":function:")[0].split(":")[-1]
@@ -900,8 +953,13 @@ def get_lambda_event_request_context(invocation_context: ApiInvocationContext):
         "requestTime": datetime.datetime.utcnow(),
         "requestTimeEpoch": int(time.time() * 1000),
     }
+
+    # set "authorizer" and "identity" event attributes from request context
+    auth_context = invocation_context.auth_context
     if auth_context:
         request_context["authorizer"] = auth_context
+    request_context["identity"].update(invocation_context.auth_identity or {})
+
     if not is_test_invoke_method(method, path):
         request_context["path"] = (f"/{stage}" if stage else "") + relative_path
         request_context["stage"] = stage
