@@ -1,8 +1,9 @@
 import json
 import logging
 import re
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Tuple
 
+from botocore.utils import InvalidArnException
 from jsonpatch import apply_patch
 from jsonpointer import JsonPointerException
 from moto.apigateway import models as apigateway_models
@@ -16,6 +17,7 @@ from localstack.services.generic_proxy import RegionBackend
 from localstack.utils import common
 from localstack.utils.aws import aws_stack
 from localstack.utils.aws.aws_responses import requests_error_response_json, requests_response
+from localstack.utils.aws.aws_stack import parse_arn
 
 LOG = logging.getLogger(__name__)
 
@@ -405,7 +407,8 @@ def update_base_path_mapping(path, data):
     mapping = ([m for m in mappings_list if m["basePath"] == base_path] or [None])[0]
     if mapping is None:
         return make_error_response(
-            "Not found: mapping for domain name %s, base path %s" % (domain_name, base_path),
+            "Not found: mapping for domain name %s, base path %s in list %s"
+            % (domain_name, base_path, mappings_list),
             code=404,
         )
 
@@ -442,6 +445,7 @@ def delete_base_path_mapping(path):
 
 
 def handle_base_path_mappings(method, path, data, headers):
+    path = urlparse.unquote(path)
     if method == "GET":
         return get_base_path_mapping(path)
     if method == "POST":
@@ -930,6 +934,7 @@ def get_rest_api_paths(rest_api_id, region_name=None):
     resource_map = {}
     for resource in resources["items"]:
         path = resource.get("path")
+        # TODO: check if this is still required in the general case (can we rely on "path" being present?)
         path = path or aws_stack.get_apigateway_path_for_resource(
             rest_api_id, resource["id"], region_name=region_name
         )
@@ -937,7 +942,7 @@ def get_rest_api_paths(rest_api_id, region_name=None):
     return resource_map
 
 
-def get_resource_for_path(path, path_map):
+def get_resource_for_path(path: str, path_map: Dict[str, Dict]) -> Tuple[str, Dict]:
     matches = []
     for api_path, details in path_map.items():
         api_path_regex = re.sub(r"\{[^\+]+\+\}", r"[^\?#]+", api_path)
@@ -975,8 +980,15 @@ def connect_api_gateway_to_sqs(gateway_name, stage_name, queue_arn, path, region
     template = APIGATEWAY_SQS_DATA_INBOUND_TEMPLATE
     resource_path = path.replace("/", "")
     region_name = region_name or aws_stack.get_region()
-    queue_name = aws_stack.sqs_queue_name(queue_arn)
-    sqs_region = aws_stack.extract_region_from_arn(queue_arn) or region_name
+
+    try:
+        arn = parse_arn(queue_arn)
+        queue_name = arn["resource"]
+        sqs_region = arn["region"]
+    except InvalidArnException:
+        queue_name = queue_arn
+        sqs_region = region_name
+
     resources[resource_path] = [
         {
             "httpMethod": "POST",
