@@ -12,12 +12,16 @@ class EC2RouteTable(GenericBaseModel):
 
     def fetch_state(self, stack_name, resources):
         client = aws_stack.connect_to_service("ec2")
-        route_tables = client.describe_route_tables(
-            Filters=[
-                {"Name": "vpc-id", "Values": [self.props["VpcId"]]},
-                {"Name": "association.main", "Values": ["false"]},
-            ]
-        )["RouteTables"]
+        tags_filters = map(
+            lambda tag: {"Name": f"tag:{tag.get('Key')}", "Values": [tag.get("Value")]},
+            self.props.get("Tags") or [],
+        )
+        filters = [
+            {"Name": "vpc-id", "Values": [self.props["VpcId"]]},
+            {"Name": "association.main", "Values": ["false"]},
+        ]
+        filters.extend(tags_filters)
+        route_tables = client.describe_route_tables(Filters=filters)["RouteTables"]
         return (route_tables or [None])[0]
 
     def get_physical_resource_id(self, attribute=None, **kwargs):
