@@ -2,14 +2,16 @@ import json
 import logging
 import uuid
 from json import JSONDecodeError
+from typing import Dict, List
 
 from localstack.utils.aws import aws_stack
+from localstack.utils.aws.aws_models import LambdaFunction
 from localstack.utils.common import first_char_to_upper
 
 LOG = logging.getLogger(__name__)
 
 
-def sqs_error_to_dead_letter_queue(queue_arn, event, error):
+def sqs_error_to_dead_letter_queue(queue_arn: str, event: Dict, error):
     client = aws_stack.connect_to_service("sqs")
     queue_url = aws_stack.get_sqs_queue_url(queue_arn)
     attrs = client.get_queue_attributes(QueueUrl=queue_url, AttributeNames=["RedrivePolicy"])
@@ -30,7 +32,7 @@ def sqs_error_to_dead_letter_queue(queue_arn, event, error):
     return _send_to_dead_letter_queue("SQS", queue_arn, target_arn, event, error)
 
 
-def sns_error_to_dead_letter_queue(sns_subscriber_arn, event, error):
+def sns_error_to_dead_letter_queue(sns_subscriber_arn: str, event: Dict, error):
     client = aws_stack.connect_to_service("sns")
     attrs = client.get_subscription_attributes(SubscriptionArn=sns_subscriber_arn)
     attrs = attrs.get("Attributes", {})
@@ -41,13 +43,13 @@ def sns_error_to_dead_letter_queue(sns_subscriber_arn, event, error):
     return _send_to_dead_letter_queue("SNS", sns_subscriber_arn, target_arn, event, error)
 
 
-def lambda_error_to_dead_letter_queue(func_details, event, error):
+def lambda_error_to_dead_letter_queue(func_details: LambdaFunction, event: Dict, error):
     dlq_arn = (func_details.dead_letter_config or {}).get("TargetArn")
     source_arn = func_details.id
     return _send_to_dead_letter_queue("Lambda", source_arn, dlq_arn, event, error)
 
 
-def _send_to_dead_letter_queue(source_type, source_arn, dlq_arn, event, error):
+def _send_to_dead_letter_queue(source_type: str, source_arn: str, dlq_arn: str, event: Dict, error):
     if not dlq_arn:
         return
     LOG.info("Sending failed execution %s to dead letter queue %s" % (source_arn, dlq_arn))
@@ -83,7 +85,7 @@ def _send_to_dead_letter_queue(source_type, source_arn, dlq_arn, event, error):
     return dlq_arn
 
 
-def _prepare_messages_to_dlq(source_arn, event, error):
+def _prepare_messages_to_dlq(source_arn: str, event: Dict, error) -> List[Dict]:
     messages = []
     custom_attrs = {
         "RequestID": {"DataType": "String", "StringValue": str(uuid.uuid4())},
@@ -133,7 +135,7 @@ def _prepare_messages_to_dlq(source_arn, event, error):
     return messages
 
 
-def message_attributes_to_upper(message_attrs):
+def message_attributes_to_upper(message_attrs: Dict) -> Dict:
     """Convert message attribute details (first characters) to upper case (e.g., StringValue, DataType)."""
     message_attrs = message_attrs or {}
     for _, attr in message_attrs.items():
