@@ -75,6 +75,11 @@ SSL_CERT_LOCK = threading.RLock()
 # user of the currently running process
 CACHED_USER = None
 
+# type definitions for JSON-serializable objects
+JsonComplexType = Union[Dict, List]
+JsonType = Union[JsonComplexType, str, int, float, bool, None]
+SerializableObj = JsonType
+
 
 class Mock(object):
     """Dummy class that can be used for mocking custom attributes."""
@@ -842,7 +847,7 @@ def merge_dicts(*dicts, **kwargs):
     return result
 
 
-def recurse_object(obj, func, path=""):
+def recurse_object(obj: JsonType, func: Callable, path: str = "") -> Any:
     """Recursively apply `func` to `obj` (may be a list, dict, or other object)."""
     obj = func(obj, path=path)
     if isinstance(obj, list):
@@ -856,13 +861,10 @@ def recurse_object(obj, func, path=""):
     return obj
 
 
-def keys_to_lower(obj, skip_children_of=None):
+def keys_to_lower(obj: JsonComplexType, skip_children_of: List[str] = None) -> JsonComplexType:
     """Recursively changes all dict keys to first character lowercase. Skip children
     of any elements whose names are contained in skip_children_of (e.g., ['Tags'])"""
-    skip_children_of = skip_children_of or []
-    skip_children_of = (
-        skip_children_of if isinstance(skip_children_of, list) else [skip_children_of]
-    )
+    skip_children_of = ensure_list(skip_children_of or [])
 
     def fix_keys(o, path="", **kwargs):
         if any([re.match(r"(^|.*\.)%s($|[.\[].*)" % k, path) for k in skip_children_of]):
@@ -877,11 +879,11 @@ def keys_to_lower(obj, skip_children_of=None):
     return result
 
 
-def camel_to_snake_case(string):
+def camel_to_snake_case(string: str) -> str:
     return re.sub(r"(?<!^)(?=[A-Z])", "_", string).replace("__", "_").lower()
 
 
-def snake_to_camel_case(string, capitalize_first=True):
+def snake_to_camel_case(string: str, capitalize_first: bool = True) -> str:
     components = string.split("_")
     start_idx = 0 if capitalize_first else 1
     components = [x.title() for x in components[start_idx:]]
@@ -892,7 +894,7 @@ def base64_to_hex(b64_string: str) -> bytes:
     return binascii.hexlify(base64.b64decode(b64_string))
 
 
-def obj_to_xml(obj) -> str:
+def obj_to_xml(obj: SerializableObj) -> str:
     """Return an XML representation of the given object (dict, list, or primitive).
     Does NOT add a common root element if the given obj is a list.
     Does NOT work for nested dict structures."""
@@ -917,7 +919,7 @@ def mktime(ts: datetime, millis: bool = False) -> int:
     return int(ts.timestamp())
 
 
-def mkdir(folder):
+def mkdir(folder: str):
     if not os.path.exists(folder):
         os.makedirs(folder, exist_ok=True)
 
@@ -944,8 +946,8 @@ def ensure_readable(file_path: str, default_perms: int = None):
         os.chmod(file_path, default_perms)
 
 
-def chown_r(path, user):
-    """Recursive chown"""
+def chown_r(path: str, user: str):
+    """Recursive chown on the given file/directory path."""
     # keep these imports here for Windows compatibility
     import grp
     import pwd
@@ -979,7 +981,7 @@ def rm_rf(path: str):
     if not path or not os.path.exists(path):
         return
     # Running the native command can be an order of magnitude faster in Alpine on Travis-CI
-    if is_alpine():
+    if is_debian():
         try:
             return run('rm -rf "%s"' % path)
         except Exception:
@@ -1046,7 +1048,7 @@ def cp_r(src: str, dst: str, rm_dest_on_conflict=False, ignore_copystat_errors=F
         shutil.copystat = copystat_orig
 
 
-def disk_usage(path: str):
+def disk_usage(path: str) -> int:
     if not os.path.exists(path):
         return 0
 
@@ -1063,7 +1065,8 @@ def disk_usage(path: str):
     return total_size
 
 
-def format_bytes(count, default="n/a"):
+def format_bytes(count: float, default: str = "n/a"):
+    """Format a bytes number as a human-readable unit, e.g., 1.3GB or 21.53MB"""
     if not is_number(count):
         return default
     cnt = float(count)
@@ -1137,33 +1140,39 @@ def download(url: str, path: str, verify_ssl=True):
         s.close()
 
 
-def parse_request_data(method, path, data=None, headers=None):
-    """Extract request data either from query string (for GET) or request body (for POST)."""
+def parse_request_data(method: str, path: str, data=None, headers=None) -> Dict:
+    """Extract request data either from query string as well as request body (e.g., for POST)."""
     result = {}
     headers = headers or {}
     content_type = headers.get("Content-Type", "")
+
+    # add query params to result
+    parsed_path = urlparse(path)
+    result.update(parse_qs(parsed_path.query))
+
+    # add params from url-encoded payload
     if method in ["POST", "PUT", "PATCH"] and (not content_type or "form-" in content_type):
         # content-type could be either "application/x-www-form-urlencoded" or "multipart/form-data"
         try:
-            result = parse_qs(to_str(data or ""))
+            params = parse_qs(to_str(data or ""))
+            result.update(params)
         except Exception:
             pass  # probably binary / JSON / non-URL encoded payload - ignore
-    if not result:
-        parsed_path = urlparse(path)
-        result = parse_qs(parsed_path.query)
+
+    # select first elements from result lists (this is assuming we are not using parameter lists!)
     result = dict([(k, v[0]) for k, v in result.items()])
     return result
 
 
-def first_char_to_lower(s):
+def first_char_to_lower(s: str) -> str:
     return s and "%s%s" % (s[0].lower(), s[1:])
 
 
-def first_char_to_upper(s):
+def first_char_to_upper(s: str) -> str:
     return s and "%s%s" % (s[0].upper(), s[1:])
 
 
-def format_number(number, decimals=2):
+def format_number(number: float, decimals: int = 2):
     # Note: interestingly, f"{number:.3g}" seems to yield incorrect results in some cases.
     # The logic below seems to be the most stable/reliable.
     result = f"{number:.{decimals}f}"
@@ -1172,7 +1181,7 @@ def format_number(number, decimals=2):
     return result
 
 
-def is_number(s):
+def is_number(s: Any) -> bool:
     try:
         float(s)  # for int, long and float
         return True
@@ -1180,38 +1189,57 @@ def is_number(s):
         return False
 
 
-def is_mac_os():
+def to_number(s: Any) -> Union[int, float]:
+    """Cast the string representation of the given object to a number (int or float), or raise ValueError."""
+    try:
+        return int(str(s))
+    except ValueError:
+        return float(str(s))
+
+
+def is_mac_os() -> bool:
     return localstack.utils.run.is_mac_os()
 
 
-def is_linux():
+def is_linux() -> bool:
     return localstack.utils.run.is_linux()
 
 
-def is_windows():
+def is_windows() -> bool:
     return platform.system().lower() == "windows"
 
 
-def is_alpine():
+def is_debian() -> bool:
+    cache_key = "_is_debian_"
     try:
         with MUTEX_CLEAN:
-            if "_is_alpine_" not in CACHE:
-                CACHE["_is_alpine_"] = False
+            if cache_key not in CACHE:
+                CACHE[cache_key] = False
                 if not os.path.exists("/etc/issue"):
                     return False
-                out = to_str(subprocess.check_output("cat /etc/issue", shell=True))
-                CACHE["_is_alpine_"] = "Alpine" in out
+                out = to_str(subprocess.check_output(["cat", "/etc/issue"]))
+                CACHE[cache_key] = "Debian" in out
     except subprocess.CalledProcessError:
         return False
-    return CACHE["_is_alpine_"]
+    return CACHE[cache_key]
 
 
-# TODO: rename to "get_os()"
-def get_arch():
+def get_arch() -> str:
+    """
+    Returns the current machine architecture
+    :return: "amd64" when x86_64, "arm64" if aarch64, platform.machine() otherwise
+    """
+    arch = platform.machine()
+    if arch == "x86_64":
+        return "amd64"
+    if arch == "aarch64":
+        return "arm64"
+    return arch
+
+
+def get_os() -> str:
     if is_mac_os():
         return "osx"
-    if is_alpine():
-        return "alpine"
     if is_linux():
         return "linux"
     if is_windows():
@@ -1219,7 +1247,7 @@ def get_arch():
     raise Exception("Unable to determine local operating system")
 
 
-def is_command_available(cmd):
+def is_command_available(cmd: str) -> bool:
     try:
         run("which %s" % cmd, print_error=False)
         return True
@@ -1227,15 +1255,15 @@ def is_command_available(cmd):
         return False
 
 
-def short_uid():
+def short_uid() -> str:
     return str(uuid.uuid4())[0:8]
 
 
-def long_uid():
+def long_uid() -> str:
     return str(uuid.uuid4())
 
 
-def parse_json_or_yaml(markup):
+def parse_json_or_yaml(markup: str) -> JsonComplexType:
     import yaml  # leave import here, to avoid breaking our Lambda tests!
 
     try:
@@ -1250,8 +1278,8 @@ def parse_json_or_yaml(markup):
                 raise
 
 
-def json_safe(item):
-    """return a copy of the given object (e.g., dict) that is safe for JSON dumping"""
+def json_safe(item: JsonType) -> JsonType:
+    """Return a copy of the given object (e.g., dict) that is safe for JSON dumping"""
     try:
         return json.loads(json.dumps(item, cls=CustomEncoder))
     except Exception:
@@ -1259,7 +1287,7 @@ def json_safe(item):
         return json.loads(json.dumps(item, cls=CustomEncoder))
 
 
-def fix_json_keys(item):
+def fix_json_keys(item: JsonType):
     """make sure the keys of a JSON are strings (not binary type or other)"""
     item_copy = item
     if isinstance(item, list):
@@ -1322,14 +1350,18 @@ def extract_from_jsonpointer_path(target, path: str, delimiter: str = "/", auto_
     return target
 
 
-def save_file(file, content, append=False):
+def save_file(file, content, append=False, permissions=None):
     mode = "a" if append else "w+"
     if not isinstance(content, six.string_types):
         mode = mode + "b"
+
+    def _opener(path, flags):
+        return os.open(path, flags, permissions)
+
     # make sure that the parent dir exsits
     mkdir(os.path.dirname(file))
     # store file contents
-    with open(file, mode) as f:
+    with open(file, mode, opener=_opener if permissions else None) as f:
         f.write(content)
         f.flush()
 
@@ -1344,12 +1376,12 @@ def load_file(file_path, default=None, mode=None):
     return result
 
 
-def get_or_create_file(file_path, content=None):
+def get_or_create_file(file_path, content=None, permissions=None):
     if os.path.exists(file_path):
         return load_file(file_path)
     content = "{}" if content is None else content
     try:
-        save_file(file_path, content)
+        save_file(file_path, content, permissions=permissions)
         return content
     except Exception:
         pass
@@ -1527,8 +1559,8 @@ def is_zip_file(content):
 
 
 def unzip(path, target_dir, overwrite=True):
-    is_in_alpine = is_alpine()
-    if is_in_alpine:
+    is_in_debian = is_debian()
+    if is_in_debian:
         # Running the native command can be an order of magnitude faster in Alpine on Travis-CI
         flags = "-o" if overwrite else ""
         flags += " -q"
@@ -1549,7 +1581,7 @@ def unzip(path, target_dir, overwrite=True):
     def _unzip_file_entry(zip_ref, file_entry, target_dir):
         """Extracts a Zipfile entry and preserves permissions"""
         out_path = os.path.join(target_dir, file_entry.filename)
-        if is_in_alpine and os.path.exists(out_path) and os.path.getsize(out_path) > 0:
+        if is_in_debian and os.path.exists(out_path) and os.path.getsize(out_path) > 0:
             # this can happen under certain circumstances if the native "unzip" command
             # fails with a non-zero exit code, yet manages to extract parts of the zip file
             return
@@ -1903,13 +1935,13 @@ def clean_cache(file_pattern=CACHE_FILE_PATTERN, last_clean_time=None, max_age=C
     return time_now
 
 
-def truncate(data, max_length=100):
+def truncate(data: str, max_length: int = 100) -> str:
     data = str(data or "")
     return ("%s..." % data[:max_length]) if len(data) > max_length else data
 
 
 # this requires that all subclasses have been imported before(!)
-def get_all_subclasses(clazz: Type):
+def get_all_subclasses(clazz: Type) -> List[Type]:
     """Recursively get all subclasses of the given class."""
     result = set()
     subs = clazz.__subclasses__()
@@ -1919,7 +1951,7 @@ def get_all_subclasses(clazz: Type):
     return result
 
 
-def parallelize(func, arr, size=None):
+def parallelize(func: Callable, arr: List, size: int = None):
     if not size:
         size = len(arr)
     if size <= 0:
@@ -1929,7 +1961,7 @@ def parallelize(func, arr, size=None):
         return pool.map(func, arr)
 
 
-def isoformat_milliseconds(t):
+def isoformat_milliseconds(t) -> str:
     try:
         return t.isoformat(timespec="milliseconds")
     except TypeError:
